@@ -1,6 +1,8 @@
 
 /* audioopmodule - Module to detect peak values in arrays */
 
+#define PY_SSIZE_T_CLEAN
+
 #include "Python.h"
 
 #if SIZEOF_INT == 4
@@ -25,7 +27,8 @@ typedef short PyInt16;
 #endif
 
 static const int maxvals[] = {0, 0x7F, 0x7FFF, 0x7FFFFF, 0x7FFFFFFF};
-static const int minvals[] = {0, -0x80, -0x8000, -0x800000, -0x80000000};
+/* -1 trick is needed on Windows to support -0x80000000 without a warning */
+static const int minvals[] = {0, -0x80, -0x8000, -0x800000, -0x7FFFFFFF-1};
 static const unsigned int masks[] = {0, 0xFF, 0xFFFF, 0xFFFFFF, 0xFFFFFFFF};
 
 static int
@@ -322,7 +325,7 @@ audioop_check_size(int size)
 }
 
 static int
-audioop_check_parameters(int len, int size)
+audioop_check_parameters(Py_ssize_t len, int size)
 {
     if (!audioop_check_size(size))
         return 0;
@@ -337,10 +340,10 @@ static PyObject *
 audioop_getsample(PyObject *self, PyObject *args)
 {
     signed char *cp;
-    int len, size, val = 0;
-    int i;
+    Py_ssize_t len, i;
+    int size, val = 0;
 
-    if ( !PyArg_ParseTuple(args, "s#ii:getsample", &cp, &len, &size, &i) )
+    if ( !PyArg_ParseTuple(args, "s#in:getsample", &cp, &len, &size, &i) )
         return 0;
     if (!audioop_check_parameters(len, size))
         return NULL;
@@ -351,15 +354,15 @@ audioop_getsample(PyObject *self, PyObject *args)
     if ( size == 1 )      val = (int)*CHARP(cp, i);
     else if ( size == 2 ) val = (int)*SHORTP(cp, i*2);
     else if ( size == 4 ) val = (int)*LONGP(cp, i*4);
-    return PyInt_FromLong(val);
+    return PyLong_FromLong(val);
 }
 
 static PyObject *
 audioop_max(PyObject *self, PyObject *args)
 {
     signed char *cp;
-    int len, size, val = 0;
-    int i;
+    Py_ssize_t len, i;
+    int size, val = 0;
     unsigned int absval, max = 0;
 
     if ( !PyArg_ParseTuple(args, "s#i:max", &cp, &len, &size) )
@@ -374,19 +377,18 @@ audioop_max(PyObject *self, PyObject *args)
         else absval = val;
         if (absval > max) max = absval;
     }
-    if (max <= INT_MAX)
-        return PyInt_FromLong(max);
-    else
-        return PyLong_FromUnsignedLong(max);
+    return PyLong_FromUnsignedLong(max);
 }
 
 static PyObject *
 audioop_minmax(PyObject *self, PyObject *args)
 {
     signed char *cp;
-    int len, size, val = 0;
-    int i;
-    int min = 0x7fffffff, max = -0x80000000;
+    Py_ssize_t len, i;
+    int size, val = 0;
+    /* -1 trick below is needed on Windows to support -0x80000000 without
+    a warning */
+    int min = 0x7fffffff, max = -0x7FFFFFFF-1;
 
     if (!PyArg_ParseTuple(args, "s#i:minmax", &cp, &len, &size))
         return NULL;
@@ -406,8 +408,8 @@ static PyObject *
 audioop_avg(PyObject *self, PyObject *args)
 {
     signed char *cp;
-    int len, size, val = 0;
-    int i;
+    Py_ssize_t len, i;
+    int size, val = 0;
     double avg = 0.0;
 
     if ( !PyArg_ParseTuple(args, "s#i:avg", &cp, &len, &size) )
@@ -424,15 +426,15 @@ audioop_avg(PyObject *self, PyObject *args)
         val = 0;
     else
         val = (int)floor(avg / (double)(len/size));
-    return PyInt_FromLong(val);
+    return PyLong_FromLong(val);
 }
 
 static PyObject *
 audioop_rms(PyObject *self, PyObject *args)
 {
     signed char *cp;
-    int len, size, val = 0;
-    int i;
+    Py_ssize_t len, i;
+    int size, val = 0;
     unsigned int res;
     double sum_squares = 0.0;
 
@@ -450,15 +452,12 @@ audioop_rms(PyObject *self, PyObject *args)
         res = 0;
     else
         res = (unsigned int)sqrt(sum_squares / (double)(len/size));
-    if (res <= INT_MAX)
-        return PyInt_FromLong(res);
-    else
-        return PyLong_FromUnsignedLong(res);
+    return PyLong_FromUnsignedLong(res);
 }
 
-static double _sum2(short *a, short *b, int len)
+static double _sum2(short *a, short *b, Py_ssize_t len)
 {
-    int i;
+    Py_ssize_t i;
     double sum = 0.0;
 
     for( i=0; i<len; i++) {
@@ -503,14 +502,14 @@ static PyObject *
 audioop_findfit(PyObject *self, PyObject *args)
 {
     short *cp1, *cp2;
-    int len1, len2;
-    int j, best_j;
+    Py_ssize_t len1, len2;
+    Py_ssize_t j, best_j;
     double aj_m1, aj_lm1;
     double sum_ri_2, sum_aij_2, sum_aij_ri, result, best_result, factor;
 
     /* Passing a short** for an 's' argument is correct only
        if the string contents is aligned for interpretation
-       as short[]. Due to the definition of PyStringObject,
+       as short[]. Due to the definition of PyBytesObject,
        this is currently (Python 2.6) the case. */
     if ( !PyArg_ParseTuple(args, "s#s#:findfit",
                            (char**)&cp1, &len1, (char**)&cp2, &len2) )
@@ -535,7 +534,7 @@ audioop_findfit(PyObject *self, PyObject *args)
     best_result = result;
     best_j = 0;
 
-    for (j=1; j<=len1-len2; j++) {
+    for ( j=1; j<=len1-len2; j++) {
         aj_m1 = (double)cp1[j-1];
         aj_lm1 = (double)cp1[j+len2-1];
 
@@ -554,7 +553,7 @@ audioop_findfit(PyObject *self, PyObject *args)
 
     factor = _sum2(cp1+best_j, cp2, len2) / sum_ri_2;
 
-    return Py_BuildValue("(if)", best_j, factor);
+    return Py_BuildValue("(nf)", best_j, factor);
 }
 
 /*
@@ -565,7 +564,7 @@ static PyObject *
 audioop_findfactor(PyObject *self, PyObject *args)
 {
     short *cp1, *cp2;
-    int len1, len2;
+    Py_ssize_t len1, len2;
     double sum_ri_2, sum_aij_ri, result;
 
     if ( !PyArg_ParseTuple(args, "s#s#:findfactor",
@@ -596,12 +595,12 @@ static PyObject *
 audioop_findmax(PyObject *self, PyObject *args)
 {
     short *cp1;
-    int len1, len2;
-    int j, best_j;
+    Py_ssize_t len1, len2;
+    Py_ssize_t j, best_j;
     double aj_m1, aj_lm1;
     double result, best_result;
 
-    if ( !PyArg_ParseTuple(args, "s#i:findmax",
+    if ( !PyArg_ParseTuple(args, "s#n:findmax",
                            (char**)&cp1, &len1, &len2) )
         return 0;
     if ( len1 & 1 ) {
@@ -620,7 +619,7 @@ audioop_findmax(PyObject *self, PyObject *args)
     best_result = result;
     best_j = 0;
 
-    for (j=1; j<=len1-len2; j++) {
+    for ( j=1; j<=len1-len2; j++) {
         aj_m1 = (double)cp1[j-1];
         aj_lm1 = (double)cp1[j+len2-1];
 
@@ -633,16 +632,16 @@ audioop_findmax(PyObject *self, PyObject *args)
 
     }
 
-    return PyInt_FromLong(best_j);
+    return PyLong_FromSsize_t(best_j);
 }
 
 static PyObject *
 audioop_avgpp(PyObject *self, PyObject *args)
 {
     signed char *cp;
-    int len, size, val = 0, prevval = 0, prevextremevalid = 0,
+    Py_ssize_t len, i;
+    int size, val = 0, prevval = 0, prevextremevalid = 0,
         prevextreme = 0;
-    int i;
     double sum = 0.0;
     unsigned int avg;
     int diff, prevdiff, nextreme = 0;
@@ -651,8 +650,8 @@ audioop_avgpp(PyObject *self, PyObject *args)
         return 0;
     if (!audioop_check_parameters(len, size))
         return NULL;
-    if (len <= size*2)
-        return PyInt_FromLong(0);
+    if (len <= size)
+        return PyLong_FromLong(0);
     if ( size == 1 )      prevval = (int)*CHARP(cp, 0);
     else if ( size == 2 ) prevval = (int)*SHORTP(cp, 0);
     else if ( size == 4 ) prevval = (int)*LONGP(cp, 0);
@@ -682,19 +681,16 @@ audioop_avgpp(PyObject *self, PyObject *args)
         avg = 0;
     else
         avg = (unsigned int)(sum / (double)nextreme);
-    if (avg <= INT_MAX)
-        return PyInt_FromLong(avg);
-    else
-        return PyLong_FromUnsignedLong(avg);
+    return PyLong_FromUnsignedLong(avg);
 }
 
 static PyObject *
 audioop_maxpp(PyObject *self, PyObject *args)
 {
     signed char *cp;
-    int len, size, val = 0, prevval = 0, prevextremevalid = 0,
+    Py_ssize_t len, i;
+    int size, val = 0, prevval = 0, prevextremevalid = 0,
         prevextreme = 0;
-    int i;
     unsigned int max = 0, extremediff;
     int diff, prevdiff;
 
@@ -703,7 +699,7 @@ audioop_maxpp(PyObject *self, PyObject *args)
     if (!audioop_check_parameters(len, size))
         return NULL;
     if (len <= size)
-        return PyInt_FromLong(0);
+        return PyLong_FromLong(0);
     if ( size == 1 )      prevval = (int)*CHARP(cp, 0);
     else if ( size == 2 ) prevval = (int)*SHORTP(cp, 0);
     else if ( size == 4 ) prevval = (int)*LONGP(cp, 0);
@@ -735,19 +731,17 @@ audioop_maxpp(PyObject *self, PyObject *args)
             prevdiff = diff;
         }
     }
-    if (max <= INT_MAX)
-        return PyInt_FromLong(max);
-    else
-        return PyLong_FromUnsignedLong(max);
+    return PyLong_FromUnsignedLong(max);
 }
 
 static PyObject *
 audioop_cross(PyObject *self, PyObject *args)
 {
     signed char *cp;
-    int len, size, val = 0;
-    int i;
-    int prevval, ncross;
+    Py_ssize_t len, i;
+    int size, val = 0;
+    int prevval;
+    Py_ssize_t ncross;
 
     if ( !PyArg_ParseTuple(args, "s#i:cross", &cp, &len, &size) )
         return 0;
@@ -763,17 +757,17 @@ audioop_cross(PyObject *self, PyObject *args)
         if ( val != prevval ) ncross++;
         prevval = val;
     }
-    return PyInt_FromLong(ncross);
+    return PyLong_FromSsize_t(ncross);
 }
 
 static PyObject *
 audioop_mul(PyObject *self, PyObject *args)
 {
     signed char *cp, *ncp;
-    int len, size, val = 0;
+    Py_ssize_t len, i;
+    int size, val = 0;
     double factor, fval, maxval, minval;
     PyObject *rv;
-    int i;
 
     if ( !PyArg_ParseTuple(args, "s#id:mul", &cp, &len, &size, &factor ) )
         return 0;
@@ -783,10 +777,10 @@ audioop_mul(PyObject *self, PyObject *args)
     maxval = (double) maxvals[size];
     minval = (double) minvals[size];
 
-    rv = PyString_FromStringAndSize(NULL, len);
+    rv = PyBytes_FromStringAndSize(NULL, len);
     if ( rv == 0 )
         return 0;
-    ncp = (signed char *)PyString_AsString(rv);
+    ncp = (signed char *)PyBytes_AsString(rv);
 
 
     for ( i=0; i < len; i += size ) {
@@ -805,29 +799,37 @@ audioop_mul(PyObject *self, PyObject *args)
 static PyObject *
 audioop_tomono(PyObject *self, PyObject *args)
 {
+    Py_buffer pcp;
     signed char *cp, *ncp;
-    int len, size, val1 = 0, val2 = 0;
+    Py_ssize_t len, i;
+    int size, val1 = 0, val2 = 0;
     double fac1, fac2, fval, maxval, minval;
     PyObject *rv;
-    int i;
 
-    if ( !PyArg_ParseTuple(args, "s#idd:tomono",
-                           &cp, &len, &size, &fac1, &fac2 ) )
+    if ( !PyArg_ParseTuple(args, "s*idd:tomono",
+                           &pcp, &size, &fac1, &fac2 ) )
         return 0;
-    if (!audioop_check_parameters(len, size))
+    cp = pcp.buf;
+    len = pcp.len;
+    if (!audioop_check_parameters(len, size)) {
+        PyBuffer_Release(&pcp);
         return NULL;
+    }
     if (((len / size) & 1) != 0) {
         PyErr_SetString(AudioopError, "not a whole number of frames");
+        PyBuffer_Release(&pcp);
         return NULL;
     }
 
     maxval = (double) maxvals[size];
     minval = (double) minvals[size];
 
-    rv = PyString_FromStringAndSize(NULL, len/2);
-    if ( rv == 0 )
+    rv = PyBytes_FromStringAndSize(NULL, len/2);
+    if ( rv == 0 ) {
+        PyBuffer_Release(&pcp);
         return 0;
-    ncp = (signed char *)PyString_AsString(rv);
+    }
+    ncp = (signed char *)PyBytes_AsString(rv);
 
 
     for ( i=0; i < len; i += size*2 ) {
@@ -843,6 +845,7 @@ audioop_tomono(PyObject *self, PyObject *args)
         else if ( size == 2 ) *SHORTP(ncp, i/2) = (short)val1;
         else if ( size == 4 ) *LONGP(ncp, i/2)= (Py_Int32)val1;
     }
+    PyBuffer_Release(&pcp);
     return rv;
 }
 
@@ -850,10 +853,10 @@ static PyObject *
 audioop_tostereo(PyObject *self, PyObject *args)
 {
     signed char *cp, *ncp;
-    int len, size, val1, val2, val = 0;
+    Py_ssize_t len, i;
+    int size, val1, val2, val = 0;
     double fac1, fac2, fval, maxval, minval;
     PyObject *rv;
-    int i;
 
     if ( !PyArg_ParseTuple(args, "s#idd:tostereo",
                            &cp, &len, &size, &fac1, &fac2 ) )
@@ -864,16 +867,16 @@ audioop_tostereo(PyObject *self, PyObject *args)
     maxval = (double) maxvals[size];
     minval = (double) minvals[size];
 
-    if (len > INT_MAX/2) {
+    if (len > PY_SSIZE_T_MAX/2) {
         PyErr_SetString(PyExc_MemoryError,
                         "not enough memory for output buffer");
         return 0;
     }
 
-    rv = PyString_FromStringAndSize(NULL, len*2);
+    rv = PyBytes_FromStringAndSize(NULL, len*2);
     if ( rv == 0 )
         return 0;
-    ncp = (signed char *)PyString_AsString(rv);
+    ncp = (signed char *)PyBytes_AsString(rv);
 
 
     for ( i=0; i < len; i += size ) {
@@ -902,9 +905,9 @@ static PyObject *
 audioop_add(PyObject *self, PyObject *args)
 {
     signed char *cp1, *cp2, *ncp;
-    int len1, len2, size, val1 = 0, val2 = 0, minval, maxval, newval;
+    Py_ssize_t len1, len2, i;
+    int size, val1 = 0, val2 = 0, minval, maxval, newval;
     PyObject *rv;
-    int i;
 
     if ( !PyArg_ParseTuple(args, "s#s#i:add",
                       &cp1, &len1, &cp2, &len2, &size ) )
@@ -919,10 +922,10 @@ audioop_add(PyObject *self, PyObject *args)
     maxval = maxvals[size];
     minval = minvals[size];
 
-    rv = PyString_FromStringAndSize(NULL, len1);
+    rv = PyBytes_FromStringAndSize(NULL, len1);
     if ( rv == 0 )
         return 0;
-    ncp = (signed char *)PyString_AsString(rv);
+    ncp = (signed char *)PyBytes_AsString(rv);
 
     for ( i=0; i < len1; i += size ) {
         if ( size == 1 )      val1 = (int)*CHARP(cp1, i);
@@ -958,11 +961,10 @@ static PyObject *
 audioop_bias(PyObject *self, PyObject *args)
 {
     signed char *cp, *ncp;
-    int len, size;
+    Py_ssize_t len, i;
+    int size, bias;
     unsigned int val = 0, mask;
     PyObject *rv;
-    int i;
-    int bias;
 
     if ( !PyArg_ParseTuple(args, "s#ii:bias",
                       &cp, &len, &size , &bias) )
@@ -971,10 +973,10 @@ audioop_bias(PyObject *self, PyObject *args)
     if (!audioop_check_parameters(len, size))
         return NULL;
 
-    rv = PyString_FromStringAndSize(NULL, len);
+    rv = PyBytes_FromStringAndSize(NULL, len);
     if ( rv == 0 )
         return 0;
-    ncp = (signed char *)PyString_AsString(rv);
+    ncp = (signed char *)PyBytes_AsString(rv);
 
     mask = masks[size];
 
@@ -999,9 +1001,9 @@ audioop_reverse(PyObject *self, PyObject *args)
 {
     signed char *cp;
     unsigned char *ncp;
-    int len, size, val = 0;
+    Py_ssize_t len, i, j;
+    int size, val = 0;
     PyObject *rv;
-    int i, j;
 
     if ( !PyArg_ParseTuple(args, "s#i:reverse",
                       &cp, &len, &size) )
@@ -1010,10 +1012,10 @@ audioop_reverse(PyObject *self, PyObject *args)
     if (!audioop_check_parameters(len, size))
         return NULL;
 
-    rv = PyString_FromStringAndSize(NULL, len);
+    rv = PyBytes_FromStringAndSize(NULL, len);
     if ( rv == 0 )
         return 0;
-    ncp = (unsigned char *)PyString_AsString(rv);
+    ncp = (unsigned char *)PyBytes_AsString(rv);
 
     for ( i=0; i < len; i += size ) {
         if ( size == 1 )      val = ((int)*CHARP(cp, i)) << 24;
@@ -1034,9 +1036,9 @@ audioop_lin2lin(PyObject *self, PyObject *args)
 {
     signed char *cp;
     unsigned char *ncp;
-    int len, size, size2, val = 0;
+    Py_ssize_t len, i, j;
+    int size, size2, val = 0;
     PyObject *rv;
-    int i, j;
 
     if ( !PyArg_ParseTuple(args, "s#ii:lin2lin",
                       &cp, &len, &size, &size2) )
@@ -1047,15 +1049,15 @@ audioop_lin2lin(PyObject *self, PyObject *args)
     if (!audioop_check_size(size2))
         return NULL;
 
-    if (len/size > INT_MAX/size2) {
+    if (len/size > PY_SSIZE_T_MAX/size2) {
         PyErr_SetString(PyExc_MemoryError,
                         "not enough memory for output buffer");
         return 0;
     }
-    rv = PyString_FromStringAndSize(NULL, (len/size)*size2);
+    rv = PyBytes_FromStringAndSize(NULL, (len/size)*size2);
     if ( rv == 0 )
         return 0;
-    ncp = (unsigned char *)PyString_AsString(rv);
+    ncp = (unsigned char *)PyBytes_AsString(rv);
 
     for ( i=0, j=0; i < len; i += size, j += size2 ) {
         if ( size == 1 )      val = ((int)*CHARP(cp, i)) << 24;
@@ -1084,7 +1086,8 @@ static PyObject *
 audioop_ratecv(PyObject *self, PyObject *args)
 {
     char *cp, *ncp;
-    int len, size, nchannels, inrate, outrate, weightA, weightB;
+    Py_ssize_t len;
+    int size, nchannels, inrate, outrate, weightA, weightB;
     int chan, d, *prev_i, *cur_i, cur_o;
     PyObject *state, *samps, *str, *rv = NULL;
     int bytes_per_frame;
@@ -1101,8 +1104,7 @@ audioop_ratecv(PyObject *self, PyObject *args)
         PyErr_SetString(AudioopError, "# of channels should be >= 1");
         return NULL;
     }
-    bytes_per_frame = size * nchannels;
-    if (bytes_per_frame / nchannels != size) {
+    if (size > INT_MAX / nchannels) {
         /* This overflow test is rigorously correct because
            both multiplicands are >= 1.  Use the argument names
            from the docs for the error msg. */
@@ -1110,6 +1112,7 @@ audioop_ratecv(PyObject *self, PyObject *args)
                         "width * nchannels too big for a C int");
         return NULL;
     }
+    bytes_per_frame = size * nchannels;
     if (weightA < 1 || weightB < 0) {
         PyErr_SetString(AudioopError,
             "weightA should be >= 1, weightB should be >= 0");
@@ -1171,7 +1174,7 @@ audioop_ratecv(PyObject *self, PyObject *args)
 
     /* str <- Space for the output buffer. */
     if (len == 0)
-        str = PyString_FromStringAndSize(NULL, 0);
+        str = PyBytes_FromStringAndSize(NULL, 0);
     else {
         /* There are len input frames, so we need (mathematically)
            ceiling(len*outrate/inrate) output frames, and each frame
@@ -1181,19 +1184,19 @@ audioop_ratecv(PyObject *self, PyObject *args)
            case ceiling(len/inrate) * outrate. */
 
         /* compute ceiling(len/inrate) without overflow */
-        int q = len > 0 ? 1 + (len - 1) / inrate : 0;
-        if (outrate > INT_MAX / q / bytes_per_frame)
+        Py_ssize_t q = len > 0 ? 1 + (len - 1) / inrate : 0;
+        if (outrate > PY_SSIZE_T_MAX / q / bytes_per_frame)
             str = NULL;
         else
-            str = PyString_FromStringAndSize(NULL,
-                                             q * outrate * bytes_per_frame);
+            str = PyBytes_FromStringAndSize(NULL,
+                                            q * outrate * bytes_per_frame);
     }
     if (str == NULL) {
         PyErr_SetString(PyExc_MemoryError,
             "not enough memory for output buffer");
         goto exit;
     }
-    ncp = PyString_AsString(str);
+    ncp = PyBytes_AsString(str);
 
     for (;;) {
         while (d < 0) {
@@ -1210,13 +1213,12 @@ audioop_ratecv(PyObject *self, PyObject *args)
                     goto exit;
                 /* We have checked before that the length
                  * of the string fits into int. */
-                len = (int)(ncp - PyString_AsString(str));
-                if (len == 0) {
-                    /*don't want to resize to zero length*/
-                    rv = PyString_FromStringAndSize("", 0);
-                    Py_DECREF(str);
-                    str = rv;
-                } else if (_PyString_Resize(&str, len) < 0)
+                len = (Py_ssize_t)(ncp - PyBytes_AsString(str));
+                rv = PyBytes_FromStringAndSize
+                    (PyBytes_AsString(str), len);
+                Py_DECREF(str);
+                str = rv;
+                if (str == NULL)
                     goto exit;
                 rv = Py_BuildValue("(O(iO))", str, d, samps);
                 Py_DECREF(samps);
@@ -1270,9 +1272,9 @@ audioop_lin2ulaw(PyObject *self, PyObject *args)
 {
     signed char *cp;
     unsigned char *ncp;
-    int len, size, val = 0;
+    Py_ssize_t len, i;
+    int size, val = 0;
     PyObject *rv;
-    int i;
 
     if ( !PyArg_ParseTuple(args, "s#i:lin2ulaw",
                            &cp, &len, &size) )
@@ -1281,10 +1283,10 @@ audioop_lin2ulaw(PyObject *self, PyObject *args)
     if (!audioop_check_parameters(len, size))
         return NULL;
 
-    rv = PyString_FromStringAndSize(NULL, len/size);
+    rv = PyBytes_FromStringAndSize(NULL, len/size);
     if ( rv == 0 )
         return 0;
-    ncp = (unsigned char *)PyString_AsString(rv);
+    ncp = (unsigned char *)PyBytes_AsString(rv);
 
     for ( i=0; i < len; i += size ) {
         if ( size == 1 )      val = ((int)*CHARP(cp, i)) << 8;
@@ -1302,9 +1304,9 @@ audioop_ulaw2lin(PyObject *self, PyObject *args)
     unsigned char *cp;
     unsigned char cval;
     signed char *ncp;
-    int len, size, val;
+    Py_ssize_t len, i;
+    int size, val;
     PyObject *rv;
-    int i;
 
     if ( !PyArg_ParseTuple(args, "s#i:ulaw2lin",
                            &cp, &len, &size) )
@@ -1313,15 +1315,15 @@ audioop_ulaw2lin(PyObject *self, PyObject *args)
     if (!audioop_check_size(size))
         return NULL;
 
-    if (len > INT_MAX/size) {
+    if (len > PY_SSIZE_T_MAX/size) {
         PyErr_SetString(PyExc_MemoryError,
                         "not enough memory for output buffer");
         return 0;
     }
-    rv = PyString_FromStringAndSize(NULL, len*size);
+    rv = PyBytes_FromStringAndSize(NULL, len*size);
     if ( rv == 0 )
         return 0;
-    ncp = (signed char *)PyString_AsString(rv);
+    ncp = (signed char *)PyBytes_AsString(rv);
 
     for ( i=0; i < len*size; i += size ) {
         cval = *cp++;
@@ -1339,9 +1341,9 @@ audioop_lin2alaw(PyObject *self, PyObject *args)
 {
     signed char *cp;
     unsigned char *ncp;
-    int len, size, val = 0;
+    Py_ssize_t len, i;
+    int size, val = 0;
     PyObject *rv;
-    int i;
 
     if ( !PyArg_ParseTuple(args, "s#i:lin2alaw",
                            &cp, &len, &size) )
@@ -1350,10 +1352,10 @@ audioop_lin2alaw(PyObject *self, PyObject *args)
     if (!audioop_check_parameters(len, size))
         return NULL;
 
-    rv = PyString_FromStringAndSize(NULL, len/size);
+    rv = PyBytes_FromStringAndSize(NULL, len/size);
     if ( rv == 0 )
         return 0;
-    ncp = (unsigned char *)PyString_AsString(rv);
+    ncp = (unsigned char *)PyBytes_AsString(rv);
 
     for ( i=0; i < len; i += size ) {
         if ( size == 1 )      val = ((int)*CHARP(cp, i)) << 8;
@@ -1371,9 +1373,9 @@ audioop_alaw2lin(PyObject *self, PyObject *args)
     unsigned char *cp;
     unsigned char cval;
     signed char *ncp;
-    int len, size, val;
+    Py_ssize_t len, i;
+    int size, val;
     PyObject *rv;
-    int i;
 
     if ( !PyArg_ParseTuple(args, "s#i:alaw2lin",
                            &cp, &len, &size) )
@@ -1382,15 +1384,15 @@ audioop_alaw2lin(PyObject *self, PyObject *args)
     if (!audioop_check_size(size))
         return NULL;
 
-    if (len > INT_MAX/size) {
+    if (len > PY_SSIZE_T_MAX/size) {
         PyErr_SetString(PyExc_MemoryError,
                         "not enough memory for output buffer");
         return 0;
     }
-    rv = PyString_FromStringAndSize(NULL, len*size);
+    rv = PyBytes_FromStringAndSize(NULL, len*size);
     if ( rv == 0 )
         return 0;
-    ncp = (signed char *)PyString_AsString(rv);
+    ncp = (signed char *)PyBytes_AsString(rv);
 
     for ( i=0; i < len*size; i += size ) {
         cval = *cp++;
@@ -1408,10 +1410,11 @@ audioop_lin2adpcm(PyObject *self, PyObject *args)
 {
     signed char *cp;
     signed char *ncp;
-    int len, size, val = 0, step, valpred, delta,
+    Py_ssize_t len, i;
+    int size, val = 0, step, valpred, delta,
         index, sign, vpdiff, diff;
     PyObject *rv, *state, *str;
-    int i, outputbuffer = 0, bufferstep;
+    int outputbuffer = 0, bufferstep;
 
     if ( !PyArg_ParseTuple(args, "s#iO:lin2adpcm",
                            &cp, &len, &size, &state) )
@@ -1420,10 +1423,10 @@ audioop_lin2adpcm(PyObject *self, PyObject *args)
     if (!audioop_check_parameters(len, size))
         return NULL;
 
-    str = PyString_FromStringAndSize(NULL, len/(size*2));
+    str = PyBytes_FromStringAndSize(NULL, len/(size*2));
     if ( str == 0 )
         return 0;
-    ncp = (signed char *)PyString_AsString(str);
+    ncp = (signed char *)PyBytes_AsString(str);
 
     /* Decode state, should have (value, step) */
     if ( state == Py_None ) {
@@ -1513,9 +1516,10 @@ audioop_adpcm2lin(PyObject *self, PyObject *args)
 {
     signed char *cp;
     signed char *ncp;
-    int len, size, valpred, step, delta, index, sign, vpdiff;
+    Py_ssize_t len, i;
+    int size, valpred, step, delta, index, sign, vpdiff;
     PyObject *rv, *str, *state;
-    int i, inputbuffer = 0, bufferstep;
+    int inputbuffer = 0, bufferstep;
 
     if ( !PyArg_ParseTuple(args, "s#iO:adpcm2lin",
                            &cp, &len, &size, &state) )
@@ -1532,15 +1536,15 @@ audioop_adpcm2lin(PyObject *self, PyObject *args)
     } else if ( !PyArg_ParseTuple(state, "ii", &valpred, &index) )
         return 0;
 
-    if (len > (INT_MAX/2)/size) {
+    if (len > (PY_SSIZE_T_MAX/2)/size) {
         PyErr_SetString(PyExc_MemoryError,
                         "not enough memory for output buffer");
         return 0;
     }
-    str = PyString_FromStringAndSize(NULL, len*size*2);
+    str = PyBytes_FromStringAndSize(NULL, len*size*2);
     if ( str == 0 )
         return 0;
-    ncp = (signed char *)PyString_AsString(str);
+    ncp = (signed char *)PyBytes_AsString(str);
 
     step = stepsizeTable[index];
     bufferstep = 0;
@@ -1629,17 +1633,31 @@ static PyMethodDef audioop_methods[] = {
     { 0,          0 }
 };
 
+
+static struct PyModuleDef audioopmodule = {
+    PyModuleDef_HEAD_INIT,
+    "audioop",
+    NULL,
+    -1,
+    audioop_methods,
+    NULL,
+    NULL,
+    NULL,
+    NULL
+};
+
 PyMODINIT_FUNC
-initaudioop(void)
+PyInit_audioop(void)
 {
     PyObject *m, *d;
-    m = Py_InitModule("audioop", audioop_methods);
+    m = PyModule_Create(&audioopmodule);
     if (m == NULL)
-        return;
+        return NULL;
     d = PyModule_GetDict(m);
     if (d == NULL)
-        return;
+        return NULL;
     AudioopError = PyErr_NewException("audioop.error", NULL, NULL);
     if (AudioopError != NULL)
          PyDict_SetItemString(d,"error",AudioopError);
+    return m;
 }
